_msgs.cpm =
  [
    /* 0  */ "ERROR: Must specify the pin.",
    /* 1  */ "ERROR: Must specify the value.",
    /* 2  */ "ERROR: Invalid CP Value.",
    /* 3  */ "ERROR: Must specify the state.",
    /* 4  */ "ERROR: Invalid state.",
    /* 5  */ "ERROR: Must specify the group.",
    /* 6  */ "ERROR: Must specify the bit.",
    /* 7  */ "ERROR: Unknown CP.",
    /* 8  */ "ERROR: Failed to change the value for {}.",
    /* 9  */ "Changed the value for {} to {}.",
    /* 10 */ "ERROR: Failed to change the state for {}.",
    /* 11 */ "Changed the state for {} to {} with Assert {}.",
    /* 12 */ "ERROR: Failed to create the group {}.",
    /* 13 */ "Created the group {}.",
    /* 14 */ "ERROR: Failed to delete the group {}.",
    /* 15 */ "Deleted the group {}.",
    /* 16 */ "ERROR: Failed to change the state for {}",
    /* 17 */ "Changed the state for {} to {}.",
    /* 18 */ "ERROR: Failed to change the value for {}.",
    /* 19 */ "Changed the value for {} to {}.",
    /* 20 */ "ERROR: Failed to add {} to group {}.",
    /* 21 */ "Added {} to group {}.",
    /* 22 */ "ERROR: Failed to remove {} from group {}.",
    /* 23 */ "Removed {} from group {}.",
    /* 24 */ "ERROR: Invalid CP Group Value.",
  ];

function CpmStatusTableRender(xml)
{
    var i;
    var d = document.getElementById('cpmstatustable');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "No data is available for this CP.";
    }
    else
    {
        var err = GetXmlData(xml, "err");

        if (err)
        {
            r += err;
        }
        else
        {
            var type = GetXmlData(xml, "t");

            r += "<table class=\"basicTable cpmSingleTable\">";

            r += "<tr><td><b>Name</b><td colspan=\"32\">" + GetXmlData(xml, "n") + "</td></tr>";
            r += "<tr><td><b>State</b><td colspan=\"32\">" + GetXmlData(xml, "s") + "</td></tr>";
            if (type)
            {
                r += "<tr><td><b>Type</b><td colspan=\"32\">" + type + "</td></tr>";
            }
            r += "<tr><td><b>Value</b><td colspan=\"32\">" + GetXmlData(xml, "e") + "</td></tr>";

            r += "<tr><td><b>Bit</b></td>"
            for (i = 31; i >= 0; i--)
            {
                r += "<td><b>";
                if (i > 9)
                {
                   r += parseInt(i / 10) + "<br/>" + (i % 10);
                }
                else
                {
                   r += i;
                }
                r += "</b></td>";
            }
            r += "</tr>";

            r += "<tr><td><b>Level</b></td>"
            for (i = 31; i >= 0; i--)
            {
            var level = GetXmlData(xml, "v" + i);
                r += "<td>";
                r += level ? level : " ";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>I/O</b></td>"
            for (i = 31; i >= 0; i--)
            {
                var io = GetXmlData(xml, "io" + i);
                r += "<td>";
                r += io ? io : " ";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>Logic</b></td>"
            for (i = 31; i >= 0; i--)
            {
                r += "<td>";
                r += (GetXmlData(xml, "l" + i)) ? "I" : "";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>Binary</b></td>"
            for (i = 31; i >= 0; i--)
            {
                var state = GetXmlData(xml, "s" + i);
                r += "<td>";
                r += (state) ? state : "x";
                r += "</td>";
            }
            r += "</tr>";

            r += "<tr><td><b>CP#</b></td>"
            for (i = 31; i >= 0; i--)
            {
                var pin = GetXmlData(xml, "sp" + i);
                r += "<td>";
                if (pin)
                {
                    pin++;

                    if (pin > 9)
                    {
                       r += parseInt(pin / 10) + "<br/>" + (pin % 10);
                    }
                    else
                    {
                       r += pin;
                    }
                }
                r += "</td>";
            }
            r += "</tr>";

            if (type)
            {
                r += "<tr><td><b>Groups</b></td><td colspan=\"32\">"
                for (i = 31; i >= 0; i--)
                {
                    var group = GetXmlData(xml, "mg" + i);
                    if (group)
                    {
                        r += group + "<br/>";
                    }
                }
                r += "</tr>";
            }

            r += "</table>";
        }
    }

    d.innerHTML = r;
}

function CpmPinsTableRender(xml)
{
    var d = document.getElementById('cpmpinstable');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "No CP data is available.";
    }
    else
    {
        r += "<table class=\"basicTable cpmMultipleTable\"><thead><tr><th>CP</th><th>Pin #</th><th>Configured As</b></th><th>State</th><th>Groups</th><th>Active In Group</th></tr></thead><tbody>";

        for (var i = 0; xml && i < xml.childNodes.length; i++)
        {
            var node = xml.childNodes[i];

            if (node.nodeName.toLowerCase() == "c")
            {
                var cp = GetXmlData(node, "n");

                r += "<tr>";
                r += "<td><a href=\"javascript:void(0)\" onclick=\"ajax_AddToHistory('CpmPinsPage', [true, 'pin=" + cp + "']); return false;\">" + cp + "</a></td>";
                r += "<td>" + GetXmlData(node, "p") + "</td>";
                r += "<td>" + GetXmlData(node, "t") + "</td>";
                r += "<td>" + GetXmlData(node, "s") + "</td>";
                r += "<td>" + GetXmlData(node, "g") + "</td>";
                var active = GetXmlData(node, "a");
                r += "<td>";
                r += (active) ? active : "&lt;available&gt;";
                r += "</td>";
                r += "</tr>";
            }
        }

        r += "</tbody></table>";
    }

    d.innerHTML = r;
}

function CpmPinsFormRenderSelectPin(xml)
{
    var r = '';

    r += "<select name=\"pin\">";

    for (var i = 0; xml && i < xml.childNodes.length; i++)
    {
        var node = xml.childNodes[i];

        r += "<option value=\"" + node.nodeName + "\"";
        r += (node.firstChild.nodeValue == "1") ? " selected" : "";
        r += ">" + node.nodeName + "</option>";
    }

    r += "</select>";

    return r;
}

function CpmPinsFormRender(xml)
{
    var d = document.getElementById('cpmpinform');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "Failed to generate form.";
    }
    else
    {
        r += "<form id=\"setform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Set</b> ";
        r += CpmPinsFormRenderSelectPin(xml);
        r += " <b>to value</b> <input type=\"text\" name=\"value\" size=\"10\" maxlength=\"10\" value=\"\" />";
        r += "&nbsp;<input onclick=\"return ajax_CpmPinsPost([true, ajaxGetFormValues('setform')]);\" type=\"submit\" name=\"set\" value=\"Submit\" />";
        r += "</form>";

        r += "<form id=\"stateform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Set</b> ";
        r += CpmPinsFormRenderSelectPin(xml);
        r += " <b>as</b> <select name=\"dir\"><option value=\"Input\">Input</option><option value=\"Output\">Output</option></select>";
        r += "&nbsp;<input name=\"low\" type=\"checkbox\" value=\"1\" unchecked />Assert Low";
        r += "&nbsp;<input onclick=\"return ajax_CpmPinsPost([true, ajaxGetFormValues('stateform')]);\" type=\"submit\" name=\"state\" value=\"Submit\" />";
        r += "</form>";
    }

    d.innerHTML = r;
}

function CpmGroupsTableRender(xml)
{
    var d = document.getElementById('cpmgroupstable');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "No CP data is available.";
    }
    else
    {
        r += "<table class=\"basicTable cpmMultipleTable\"><thead><tr><th>Group Name</th><th>State</th><th>CP Info</th></tr></thead><tbody>";

        for (var i = 0; xml && i < xml.childNodes.length; i++)
        {
            var node = xml.childNodes[i];

            if (node.nodeName.toLowerCase() == "g")
            {
                var group = GetXmlData(node, "n");

                r += "<tr>";
                r += "<td><a href=\"javascript:void(0)\" onclick=\"return ajax_AddToHistory('CpmGroupsPage', [true, 'group=" + encodeURIComponent(group) + "']);\">" + group + "</a></td>";
                r += "<td>" + GetXmlData(node, "s") + "</td>";
                r += "<td>" + GetXmlData(node, "c") + "</td>";
                r += "</tr>";
            }
        }

        r += "</tbody></table>";
    }

    d.innerHTML = r;
}

function CpmGroupsFormRenderSelectGroup(xml)
{
    var r = '';

    r += "<select name=\"group\">";

    for (var i = 0; xml && i < xml.childNodes.length; i++)
    {
        var g = xml.getElementsByTagName("g" + i);
        if (!g || !g.length)
        {
            break;
        }
        var gv = g[0].firstChild.nodeValue;

        var gs = xml.getElementsByTagName("gs" + i);
        var gsv = gs[0].firstChild.nodeValue;

        r += "<option value=\"" + gv + "\"";
        r += (gsv == "1") ? " selected" : "";
        r += ">" + gv + "</option>";
    }

    r += "</select>";

    return r;
}

function CpmGroupsFormRenderSelectPin(xml)
{
    var r = '';

    r += "<select name=\"pin\">";

    for (var i = 0; xml && i < xml.childNodes.length; i++)
    {
        var cp = xml.getElementsByTagName("CP" + (i + 1));
        if (!cp || !cp.length)
        {
            break;
        }
        var cpv = cp[0].nodeName;

        r += "<option value=\"" + cpv + "\">" + cpv + "</option>";
    }

    r += "</select>";

    return r;
}

function CpmGroupsFormRender(xml)
{
    var i;
    var d = document.getElementById('cpmgroupform');
    d.innerHTML = '';

    var r = '';

    if (!xml || !xml.hasChildNodes())
    {
        r += "Failed to generate form.";
    }
    else
    {
        r += "<form id=\"createform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Create Group:</b> <input type=\"text\" name=\"group\" size=\"16\" maxlength=\"16\" value=\"\" />";
        r += "&nbsp;<input onclick=\"return ajax_CpmGroupsPost([true, ajaxGetFormValues('createform')]);\" type=\"submit\" name=\"create\" value=\"Submit\" />";
        r += "</form>";

        r += "<form id=\"deleteform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Delete Group:</b> ";
        r += CpmGroupsFormRenderSelectGroup(xml);
        r += "&nbsp;<input onclick=\"return ajax_CpmGroupsPost([true, ajaxGetFormValues('deleteform')]);\" type=\"submit\" name=\"delete\" value=\"Submit\" />";
        r += "</form>";

        r += "<form id=\"stateform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Set</b> ";
        r += CpmGroupsFormRenderSelectGroup(xml);
        r += " <b>state to</b> ";
        r += "<select name=\"status\"><option value=\"Enabled\">Enabled</option><option value=\"Disabled\">Disabled</option></select>";
        r += "&nbsp;<input onclick=\"return ajax_CpmGroupsPost([true, ajaxGetFormValues('stateform')]);\" type=\"submit\" name=\"state\" value=\"Submit\" />";
        r += "</form>";

        r += "<form id=\"setform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Set</b> ";
        r += CpmGroupsFormRenderSelectGroup(xml);
        r += " <b>to value</b> <input type=\"text\" name=\"value\" size=\"10\" maxlength=\"10\" value=\"\" />";
        r += "&nbsp;<input onclick=\"return ajax_CpmGroupsPost([true, ajaxGetFormValues('setform')]);\" type=\"submit\" name=\"set\" value=\"Submit\" />";
        r += "</form>";

        r += "<form id=\"addform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Add</b> ";
        r += CpmGroupsFormRenderSelectPin(xml);
        r += " <b>to</b> ";
        r += CpmGroupsFormRenderSelectGroup(xml);
        r += " <b>at bit</b> ";
        r += "<select name=\"bit\">";
        r += "<option value=\"Next\">Next</option>";
        for (i = 0; i < 32; i++)
        {
            r += "<option value=\"" + i + "\">" + i + "</option>";
        }
        r += "</select>";
        r += "&nbsp;<input onclick=\"return ajax_CpmGroupsPost([true, ajaxGetFormValues('addform')]);\" type=\"submit\" name=\"add\" value=\"Submit\" />";
        r += "</form>";

        r += "<form id=\"removeform\" style=\"margin-top: 10px;\" method=\"post\">";
        r += "<b>Remove</b> "
        r += CpmGroupsFormRenderSelectPin(xml);
        r += " <b>from</b> ";
        r += CpmGroupsFormRenderSelectGroup(xml);
        r += "&nbsp;<input onclick=\"return ajax_CpmGroupsPost([true, ajaxGetFormValues('removeform')]);\" type=\"submit\" name=\"remove\" value=\"Submit\" />";
        r += "</form>";
    }

    d.innerHTML = r;
}

function SetCpmNav(xml)
{
    SetNav(xml, false, true);
}
