_msgs.fs =
  [
    /* 0  */ "The Filesystem has been compacted.",
    /* 1  */ "The Filesystem has been formatted.",
    /* 2  */ "ERROR: Current working directory is unknown.",
    /* 3  */ "ERROR: Must specify the directory name.",
    /* 4  */ "ERROR: Invalid directory name.",
    /* 5  */ "ERROR: Must specify the file name.",
    /* 6  */ "ERROR: Invalid file name.",
    /* 7  */ "ERROR: Unknown file name.",
    /* 8  */ "ERROR: Must specify the source file.",
    /* 9  */ "ERROR: Must specify the destination file.",
    /* 10 */ "ERROR: Invalid source file name.",
    /* 11 */ "ERROR: Invalid destination file name.",
    /* 12 */ "ERROR: Invalid source file name.",
    /* 13 */ "ERROR: Invalid destination file name.",
    /* 14 */ "ERROR: Failed to open source file.",
    /* 15 */ "ERROR: Failed to open destination file.",
    /* 16 */ "ERROR: Failed to copy the file.",
    /* 17 */ "ERROR: Failed to move the file.",
    /* 18 */ "ERROR: Must specify the TFTP Action.",
    /* 19 */ "ERROR: Invalid TFTP Action.",
    /* 20 */ "ERROR: Must specify the TFTP Mode.",
    /* 21 */ "ERROR: Invalid TFTP Mode.",
    /* 22 */ "ERROR: Must specify the TFTP Host.",
    /* 23 */ "ERROR: Invalid TFTP Port. Valid range is from 1 to 65535.",
    /* 24 */ "ERROR: Must specify the Local File.",
    /* 25 */ "ERROR: Invalid Local File name.",
    /* 26 */ "ERROR: Must specify the Remote File.",
    /* 27 */ "Downloading firmware image.  Do not power off or reset system!",
    /* 28 */ "System will reboot automatically after the new image is programmed.",
    /* 29 */ "ERROR: Failed to read the directory.",
    /* 30 */ "ERROR: Unknown path.",
    /* 31 */ "ERROR: Unknown current working directory.",
    /* 32 */ "ERROR: Invalid file name.",
    /* 33 */ "ERROR: No directory specified.",
    /* 34 */ "ERROR: Failed to delete the directory {}.",
    /* 35 */ "The directory {} has been deleted.",
    /* 36 */ "ERROR: Failed to delete the file {}.",
    /* 37 */ "The file {} has been deleted.",
    /* 38 */ "ERROR: Failed to create the directory {}.",
    /* 39 */ "Created the directory {}.",
    /* 40 */ "ERROR: {} already exists.",
    /* 41 */ "Created the file {}.",
    /* 42 */ "ERROR: Failed to create the file {}.",
    /* 43 */ "ERROR: Failed to upload the file {}.",
    /* 44 */ "Uploaded the file {}.",
    /* 45 */ "Copied the file {} to {}.",
    /* 46 */ "Moved the file {} to {}.",
    /* 47 */ "TFTP Put success of {}.",
    /* 48 */ "TFTP Get success of {}.",
    /* 49 */ "ERROR: TFTP failed - {}",
  ];

function PrintFsBrowseResults(xml) { PrintMsg(xml, 'fsresults'); }
function PrintFsCreateResults(xml) { PrintMsg(xml, 'fscreate'); }
function PrintFsUploadResults(xml) { PrintMsg(xml, 'fsupload'); }
function PrintFsCopyResults(xml) { PrintMsg(xml, 'fscopy'); }
function PrintFsMoveResults(xml) { PrintMsg(xml, 'fsmove'); }
function PrintFsTFtpResults(xml) { PrintMsg(xml, 'fstftp'); }

function FsDirCompare(a, b)
{
    if (a > b) return 1;
    if (a < b) return -1;
    return 0;
}

function FsFileCompare(a, b)
{
    if (a[0] > b[0]) return 1;
    if (a[0] < b[0]) return -1;
    return 0;
}

function FsBasename(path)
{
    if (path == '/') return path;
    var r = path.match(/[\/|\\]([^\\\/]+)$/);
    if (!r) return '';
    return r[1];
}

function FsDirname(path)
{
    if (path == '/') return path;
    var r = path.match(/(.*)[\/|\\][^\\\/]+$/);
    if (!r) return '';
    if (!r[1]) return '/';
    return r[1];
}

function FsBrowseRender(xml)
{
    if (!xml || !xml.hasChildNodes())
    {
        alert("ERROR: No Filesystem information available.");
        return;
    }

    var cwd = xml.getElementsByTagName('cwd');
    if (!cwd)
    {
        alert("ERROR: Unable to determine the current directory.");
        return;
    }

    cwd = cwd[0].firstChild.nodeValue;

    var i;
    var j;
    var r;

    var b = document.getElementById('breadcrumbs');
    b.innerHTML = '';

    var d = document.getElementById('fsdata');
    d.innerHTML = '';

    r = '&nbsp;&nbsp;/';

    if (cwd != '/')
    {
        var elems = cwd.split('/');
        for (i = 0; i < elems.length; i++)
        {
            if (elems[i])
            {
                var ldir = '';

                for (j = 0; j <= i; j++)
                {
                    ldir += '/' + elems[j];
                }

                r += "&nbsp;<a href=\"javascript:void(0)\" onclick=\"return ajax_AddToHistory('FsBrowsePage', [true, 'dir=" + encodeURIComponent(ldir) + "']);\">" + elems[i] + "</a>&nbsp;/";
            }
        }
    }

    b.innerHTML = r;

    var dirs = new Array();
    var files = new Array();

    for (i = 0; i < xml.childNodes.length; i++)
    {
        var node = xml.childNodes[i];

        if (node.nodeName.toLowerCase() == "d")
        {
            dirs.push(node.firstChild.nodeValue);
        }
    }

    dirs.sort(FsDirCompare);

    for (var i = 0; i < xml.childNodes.length; i++)
    {
        var f = xml.getElementsByTagName("f" + i);
        if (!f || !f.length)
        {
            break;
        }
        var fv = f[0].firstChild.nodeValue;

        var fl = xml.getElementsByTagName("fl" + i);
        var flv = fl[0].firstChild.nodeValue;

        files.push(new Array(fv, flv));
    }

    files.sort(FsFileCompare);

    var r = '';

    r += "<table>";

    var root = false;
    if (cwd == '/')
    {
        root = true;
    }

    if (!root)
    {
        r += "<tr><td>&nbsp;&nbsp;&nbsp;<img src=\"/config/folder.gif\" /></td><td><a href=\"javascript:void(0)\" onclick=\"return ajax_AddToHistory('FsBrowsePage', [true, 'dir=" + encodeURIComponent(FsDirname(cwd)) + "']);\">&lt;up&gt;</a></td><td></td></tr>";
    }

    for (i = 0; i < dirs.length; i++)
    {
        r += "<tr>";
        r += "<td>&nbsp;&nbsp;&nbsp;<img src=\"/config/folder.gif\" />&nbsp;&nbsp;<a href=\"javascript:void(0)\" onclick=\"if (confirm('Are you SURE you want to delete the directory " + dirs[i] + "?')) ajax_FsBrowseClean([true, 'dir=" + encodeURIComponent(cwd) + "&amp;path=";
        r += (root) ? encodeURIComponent(cwd + dirs[i]) : encodeURIComponent(cwd + '/' + dirs[i]);
        r += "&amp;action=deletedir']); return false;\"><img src=\"/config/remove.gif\" /></a></td>";
        r += "<td><a href=\"javascript:void(0)\" onclick=\"return ajax_AddToHistory('FsBrowsePage', [true, 'dir=";
        r += (root) ? encodeURIComponent(cwd + dirs[i]) : encodeURIComponent(cwd + '/' + dirs[i]);
        r += "']);\">" + dirs[i] + "</a></td><td></td>";
        r += "</tr>";
    }

    for (i = 0; i < files.length; i++)
    {
        r += "<tr>";
        r += "<td>&nbsp;&nbsp;&nbsp;<img src=\"/config/file.gif\" />&nbsp;&nbsp;<a href=\"javascript:void(0)\" onclick=\"if (confirm('Are you SURE you want to delete the file " + files[i][0] + "?')) ajax_FsBrowseClean([true, 'dir=" + encodeURIComponent(cwd) + "&amp;path=";
        r += (root) ? encodeURIComponent(cwd + files[i][0]) : encodeURIComponent(cwd + '/' + files[i][0]);
        r += "&amp;action=deletefile']); return false;\"><img src=\"/config/remove.gif\" /></a></td>";
        r += "<td><a href=\"/fs?file=";
        r += (root) ? encodeURIComponent(cwd + files[i][0]) : encodeURIComponent(cwd + '/' + files[i][0]);
        r += "\" target=\"_blank\">" + files[i][0] + "</a></td><td>" + files[i][1] + "</td>";
        r += "</tr>";
    }

    r += "</table>";

    d.innerHTML = r;
}

function FsUploadClearForm(xml)
{
    var p = document.getElementById('uploadform');
    var oldInput = document.getElementById('fsuploadfile');
    var newInput = document.createElement('input');
    newInput.id = 'fsuploadfile';
    newInput.name = 'file';
    newInput.type = 'file';
    newInput.size = '32';
    newInput.maxlength = '255';
    p.replaceChild(newInput, oldInput);
}

function SetFsNav(xml)
{
    SetNav(xml, false, true);
}
